import numpy as np

def MA_smoothing(data, avg_len, smooth_type='middle'):
    data = np.array(data)
    opt = data.copy()
    if smooth_type == 'middle':
        if avg_len % 2 == 0:
            print('Warning: avg_len ({}) is suggest to be odd when smooth_type is \'middle\'.'.format(avg_len))
        avg_len = int(avg_len/2)

        for i in range(len(data)):
            start_i, end_i = max(0, i - avg_len), min(len(data), i + avg_len + 1)
            opt[i] = np.mean(data[start_i: end_i])

    elif smooth_type == 'start':
        for i in range(len(data)):
            end_i = min(len(data), i + avg_len + 1)
            opt[i] = np.mean(data[i: end_i])
        
    elif smooth_type == 'end':
        for i in range(len(data)):
            start_i = max(0, i - avg_len)
            opt[i] = np.mean(data[start_i: i])
        
    else:
        raise ValueError('smooth_type must be set to \'middle\', \'start\' or \'end\'.')

    return opt

def _demo():
    print('This is a demo of MA_smoothing.')
    t = np.linspace(-4*np.pi, 4*np.pi, 400)
    noisy_sin_t = np.sin(t) + np.random.normal(0, 1, t.shape) * 0.1
    smooth_sin_t = MA_smoothing(noisy_sin_t, 11)

    import matplotlib.pyplot as plt
    plt.subplot(211)
    plt.plot(noisy_sin_t, label='noisy sine')
    plt.subplot(212)
    plt.plot(smooth_sin_t, label='smooth sine')
    plt.show()

if __name__ == '__main__':
    _demo()